/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MontageInterface.h"
#import "MIWSSelection.h"
#import "MISnapper.h"

@class MIObject;
@class MIMontageDocument;
@class MIObjectRep;
@class MITopLevelContainerRep;
@class MIRuler;
@class MIDockBar;

enum MIGridFlags
{
	MIGlobalGrid = 1,
	MILocalGrid = 2,
	MIObjectsGrid = 4,
	MIShowGrid = 8
};

@interface MIMainViewController : NSWindowController
{
	IBOutlet id 		fMainScrollView;
	IBOutlet id 		fTopLevelContainerView;
	IBOutlet id			fRuler;
	IBOutlet id			fPanelLayoutManager;
	MIObject			*fTLC;
	MIWSSelection*		fSelection;
@public
	UInt32				fVZoomIndex;
	UInt32				fHZoomIndex;
@protected
	unsigned int		fGridFlags;
	MISnapper*			fSnapper;
}

- (void)setRepresentedDocument:(MIMontageDocument*)document;
- (MIMontageDocument*)representedDocument;

- (id)panelLayoutManager;

- (void)setTLC:(MIObject*)tlc;
- (MIObject*)getTLC;

- (void)setTopLevelContainerView:(MITopLevelContainerRep*)newTLC;
- (MITopLevelContainerRep*)topLevelContainerView;

- (NSScrollView*)mainScrollView;

- (MIDockBar*)mainDockBar;

- (MIRuler*)ruler;

- (void)nudgeSelection:(unichar)key withModifiers:(unsigned int)modifiers;

- (MIWSSelection*)selection;
- (void)clearSelection;
//- (BOOL)setSelection:(NSArray*)newSelectionArray;
- (NSEnumerator*)selectionObjectEnumerator;
- (BOOL)addObjectToSelection:(MIObjectRep*)anObjectRep;
- (BOOL)removeObjectFromSelection:(MIObjectRep*)anObjectRep;
- (BOOL)isObjectInSelection:(MIObjectRep*)anObjectRep;
- (void)setSelectionEndxTime:(ArmeTime)endxTime;
- (void)setSelectionStartTime:(ArmeTime)startTime;
- (void)selectionSetTimeSliceType;
- (void)selectionSetObjectsType;
- (void)selectAll;

/*! @function setHZoom changes the horizontal zoom with a new index.
    This index is a position in the gHZoomPNValues array. Each value
	in this array represents a zoom value. A positive value represents
	frames per pixel, and a negative value represents -(pixels per frame).
	This, a zoom pnvalue of 1000 means 1000 frames/pixels while -10 represents
	10 pixels/frame. Obviously, -1 and 1 are the same zoom values.*/
- (void)setHZoom:(UInt32)hZoomIndex;
/*! @function setVZoom changes the vertical zoom with a new index
    This index is a position in the gVZoomPNValues array. Each value
	in this array represents a zoom value. A positive value represents
	vpos per pixel, and a negative value represents -(pixels per vpos).
	This, a zoom pnvalue of 1000 means 1000 vpos/pixels while -10 represents
	10 pixels/vpos. Obviously, -1 and 1 are the same zoom values.*/
- (void)setVZoom:(UInt32)vZoomIndex;
/*! @function hZoomIn increments the h zoom to the previous gHZoomPNValues value*/
- (void)hZoomIn;
/*! @function hZoomOut decrements the h zoom to the next gHZoomPNValues value*/
- (void)hZoomOut;
/*! @function hZoomIn increments the v zoom to the previous gVZoomPNValues value*/
- (void)vZoomIn;
/*! @function hZoomOut decrements the v zoom to the next gVZoomPNValues value*/
- (void)vZoomOut;

/*! @function vZoomIndex Returns the vertical zoom index (index in the gVZoomPNValues array)*/
- (UInt32)vZoomIndex;
/*! @function hZoomIndex Returns the horizontal zoom index (index in the gHZoomPNValues array)*/
- (UInt32)hZoomIndex;

/*! @function getNbHZoomIndices Returns the number of H zoom values */
+ (UInt32)getNbHZoomIndices;
/*! @function getNbVZoomIndices Returns the number of V zoom values */
+ (UInt32)getNbVZoomIndices;
/*! @function getDefaultHZoomIndex Returns the default horizontal zoom as an index in the gHZoomPNValues array*/
+ (UInt32)getDefaultHZoomIndex;
/*! @function getDefaultVZoomIndex Returns the default vertical zoom as an index in the gHZoomPNValues array*/
+ (UInt32)getDefaultVZoomIndex;

/*! @function */
+ (SInt32)getHZoomPNValueFromIndex:(UInt32)hZoomIndex;
/*! @function */
+ (SInt32)getVZoomPNValueFromIndex:(UInt32)vZoomIndex;
/*! @function */
- (SInt32)getCurrentHZoomPNValue;
/*! @function */
- (SInt32)getCurrentVZoomPNValue;
/*! @function */
+ (double)getFramesPerPixelFromHZoomIndex:(UInt32)hZoomIndex;
/*! @function */
+ (double)getVPosPerPixelFromVZoomIndex:(SInt32)vZoomIndex;
/*! @function */
- (double)getCurrentFramesPerPixel;
/*! @function */
- (double)getCurrentVPosPerPixel;

-(MISnapper*)snapper;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// conversion functions model coords <-> tlc rep coords (pixel)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*! @function pixelToArmeTime */
- (ArmeTime)pixelToArmeTime:(float)pixels;
/*! @function armeTimeToPixel */
- (float)armeTimeToPixel:(ArmeTime)time;
- (double)doubleTimeToPixel:(double)time;
- (double)pixelToDoubleTime:(float)pixels;


/*! @function pixelToMIVPos converts pixels to a vertical position the model uses taking zoom into account */
- (MIVPos)pixelToMIVPos:(float)pixels;

/*! @function MIVPosToPixel converts MIVpos that the model uses to pixels taking zoom into account */
- (float)MIVPosToPixel:(MIVPos)vpos;

- (MIPoint)NSPointToMIPoint:(NSPoint)pt;

- (NSPoint)MIPointToNSPoint:(MIPoint)mipt;

- (MISize)NSSizeToMISize:(NSSize)size;

- (NSSize)MISizeToNSSize:(MISize)misize;

- (MIRect)NSRectToMIRect:(NSRect)rc;

- (NSRect)MIRectToNSRect:(MIRect)mirc;

/*! @function redraw This function is called by the document when the model has changed and needs to be redrawn */
/* $$$STEPHAN_ADVISE : please think about the notification model and tell me what you think... for example : 
	what happens if the background tasks notifies a model change in the middle of a tracker ? */
//- (void)redraw;

- (void)TLCScrolled:(NSNotification*)theNotification;

- (void)updatePlayBar;
- (void)updateLocators;

- (void)updateObjectRepresentations;
- (MIObjectRep*)findRepresentationOfObject:(MIObject*)anObject;

- (unsigned int)gridFlags;
- (void)setGridFlags:(unsigned int)gridFlags;

- (void)executeCommand:(NSString*)commandName;
- (BOOL)canMenuItemActionBePerformedOnCurrentSelection:(NSMenuItem*)menuItem;

- (void)setLocatorsFromSelection;

- (IBAction)import:(id)sender;
- (void)importFilesUsingImportPanelIfNeeded:(NSArray*)files importSelector:(SEL)selector importTarget:(id)target contextDictionary:(NSMutableDictionary*)contextDict;
- (BOOL)unwantedFiles:(NSArray*)files;

// debug stuff
- (void)audioClipInspector;
- (void)debugAudioRecorder;
@end
